# Enfold License Manager
# Copyright(C), 2004-2008, Enfold Systems, Inc. - ALL RIGHTS RESERVED

# Enfold Systems, Inc.
# 4617 Montrose Blvd., Suite C215
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com

# $Id: products.py 9811 2009-04-21 00:18:36Z sidnei $
# $HeadURL: https://svn.enfoldsystems.com/internal/python-lib/trunk/enfold/enfold/runtime/products.py $

import os
import logging
from warnings import warn
from enfold.runtime.license import licenseStorage, LicenseException

pkg_name = 'Enfold License Manager'
logger = logging.getLogger(pkg_name)
log = logger.info
debug = logger.debug

def handle_license_error(text):
    # Always log with logging package at level info, no matter what
    # happens.
    log(text)
    try:
        # Try and write to the Windows event log.  
        import win32evtlogutil
        try:
            win32evtlogutil.AddSourceToRegistry(pkg_name)
        except:
            pass
        # A failure to add a source to the registry is not
        # critical according to Mark. So we ignore it and try to
        # report the event anyway.
        win32evtlogutil.ReportEvent(pkg_name, 1, strings=(text,))
    except ImportError:
        # Linux?
        pass
    except:
        log("Failed to write the license error to the Event Log.")

def getPloneDesktopLicenses():
    """ Figure out if we can find a valid Plone Desktop license
    """
    warn('getPloneDesktopLicenses is deprecated, please use '
         'getDesktopLicenseData instead.', DeprecationWarning, 2)
    return getDesktopLicenseData()

def getEESLicenses(report_error=True):
    """ Figure out if we can find a valid ES license
    """
    warn('getEESLicenses is deprecated, please use '
         'getServerLicense instead.', DeprecationWarning, 2)
    return getServerLicense(report_error=report_error)


def callLicenseMethod(m, report_error, errors, *args, **kw):
    try:
        return m(*args, **kw)
    except LicenseException, details:
        errors.append(details)
    except:
        # Should never get here.
        if report_error:
            logger.exception('Exception calling %s (args: %s, kw: %s).',
                             repr(m), repr(args), repr(kw))
            handle_license_error(
                "Failed to extract license information - please "
                "contact Enfold Systems for assistance")

def getDesktopLicenseSuffix(default=' 4'):
    # Enfold Desktop is always branched along with LicenseManager. So
    # the suffix is static (at least for now).
    return default

# ES 3.0 needed a new license suffix.  For a while it was simply '3' - but
# we may move to a 'year' suffix if our license model moves that way.
def getServerLicenseSuffix(default=' 4'):
    # Good enough at the moment is checking the CMFPlone version - but it
    # will need to change for a year-based system.
    version_tuple = None
    try:
        from Products.CMFPlone.utils import getFSVersionTuple
        version_tuple = getFSVersionTuple()
    except ImportError:
        pass
    try:
        # Try looking for eggs in known-good locations
        import pkg_resources

        buildout_dirs = []
        runtime_path = pkg_resources.resource_filename('enfold.runtime', '')
        parts = os.path.normpath(
            os.path.abspath(runtime_path)).split(os.path.sep)
        while parts:
            fake = os.path.sep.join(parts + ['fake-eggs'])
            if os.path.exists(fake):
                for name in os.listdir(fake):
                    buildout_dirs.append(os.path.join(fake, name))
                buildout_dirs.append(os.path.join(os.path.dirname(fake),
                                                  'buildout-cache', 'eggs'))
                break
            parts.pop(-1)
        if buildout_dirs:
            ws = pkg_resources.working_set
            dists, errors = ws.find_plugins(
                pkg_resources.Environment(buildout_dirs))
            for dist in dists:
                if dist.project_name.lower() == 'plone':
                    version_tuple = tuple(map(int, dist.parsed_version[:-1]))
                    break
    except ImportError:
        pass
    if version_tuple is None:
        return default
    elif version_tuple >= (3, 2):
        return " 5"
    elif version_tuple >= (3, 0):
        return " 4"
    elif version_tuple > (2, 5):
        return " 3"
    else:
        return ""
    
def safeGetLicenses(names, report_error):
    good = []
    errors = []
    storage = licenseStorage()
    licenses = callLicenseMethod(storage.getLicenses, report_error, errors)
    if licenses is not None:
        for name in names:
            lic = licenses.get(name)
            if lic is None:
                continue
            expired = callLicenseMethod(lic.checkExpired, report_error, errors)
            if expired or expired is None:
                continue
            if not lic.canExpire():
                info = 'unlimited'
            else:
                info = "%s day(s) left" % lic.daysLeft()
            debug("Found license key: %s, %s.", name, info)
            good.append(lic)
    if not good:
        errors.append('No license has been found for: %s' % ', '.join(names))
    return good, errors


def getDesktopLicenseData(report_error=True):
    """Figure out how many desktop seats we have available
    """
    suffix = getDesktopLicenseSuffix()
    names = ("Enfold Desktop" + suffix,)
    good, errors = safeGetLicenses(names, report_error)

    if good:
        # Licenses should have a '__cmp__' method which does know what
        # means for a license to be the 'longest' (usually on daysLeft())
        good.sort()
        lic = good[-1]
        daysLeft = lic.daysLeft()
        expires = lic.canExpire()
        info = '%s days' % daysLeft
        if not expires:
            info = 'unlimited'
        debug("Using license key '%s' (%s) for Enfold Desktop.", lic.getName(), info)

        # pass daysLeft back, assuming data could have a key called
        # daysLeft, we'll pass that back in a 'data' dictionary
        return {'data': lic.getData() or {},
                'daysLeft': daysLeft,
                'expires': expires}

    if report_error:
        for error in errors:
            # if there was a non-License Exception raised
            handle_license_error(
                "The Enfold Desktop Server Components have not been "
                "licensed (%s). Users will see Ads until you install "
                "an appropriate license. Please see "
                "http://www.enfoldsystems.com/Licensing for "
                "more information." % (error,))
    return {}


def getServerLicenseData(report_error=True):
    """ Figure out the Enfold Server license information
    """
    lic = getServerLicense(report_error=report_error)
    if not lic:
        return {}
    return {'data': lic.getData(),
            'daysLeft': lic.daysLeft(),
            'expires': lic.canExpire()}

def getServerLicense(report_error=True):
    """ Figure out if we can find a valid Enfold Server license
    """
    suffix = getServerLicenseSuffix()
    if suffix:
        names = ("Enfold Server" + suffix,)
    else:
        # old style
        names = ("Enfold Server", "Enfold Enterprise Server")
    good, errors = safeGetLicenses(names, report_error)

    if good:
        # Licenses should have a '__cmp__' method which does know what
        # means for a license to be the 'longest' (usually on daysLeft())
        good.sort()
        lic = good[-1]
        info = '%s days' % lic.daysLeft()
        if not lic.canExpire():
            info = 'unlimited'
        debug("Using license key '%s', %s "
              "for Enfold Server.", lic.getName(), info)
        return lic

    if report_error:
        for error in errors:
            # if there was a non-License Exception raised
            handle_license_error(
                "The appropriate license keys have not been "
                "found or have expired (%s). Please see "
                "http://www.enfoldsystems.com/Licensing for "
                "more information. The software will not work"
                "until you fix this situation." % (error,))

        try:
            import win32api
            hdl = win32api.GetCurrentProcess()
            win32api.TerminateProcess(hdl, 0)
        except ImportError:
            import signal
            # Most likely Linux. Send SIGKILL?
            os.kill(os.getpid(), signal.SIGKILL)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    print getDesktopLicenseData()
    print getServerLicenseData()
